/**
 * Written by Dvaid Smith
 * (c) 2008 ATNF CSIRO
 * 
 * This program scans the directories given on the command line for any pulsar observations
 * For every observation it finds, it checks that it exists in the database
 * If an observation exists on disk but not in the database, it is added
 * If an observation is in the database but not on disk, then we delete it
 * 
 * An observation can be in many forms
 * 1) A single PSRFITS archive ending in rf or cf
 * 2) A set of files in a CPSR2 directory ending with cfb and beginning with either m, n or o ( 1 file for each sub integration)
 * 3) If an observation is in a directory that ends with _R then it is a calibration
 * 4) If the fits file has a pulsar src_name of CAL then it is also a calibration file (check with dick about this)
 **/


#include <iostream>
#include <vector>
#include <string>
#include <dirutil.h>
#include <strutil.h>
#include <boost/regex.hpp>
#include "psrdb.h"



using std::cerr;
using std::cout;
using std::endl;
using std::vector;
using std::string;
using mysqlpp::Connection;
using mysqlpp::Query;
using mysqlpp::Exception;


const char *database = "psrchive";
const char *db_username = "psrdba";
const char *db_password = "lighthouse";
const char *db_hostname = "localhost";

psrdb db;




/// This function creates an ObservationEntry object that represents
/// the files given

ObservationEntry MakeObservationEntry( const vector<string> &files )
{
  ObservationEntry entry;
  
  string example_datafile = basename( files.front() );
  
  string full_path = pathname( files.front() );
  string stripped_path = full_path.substr( 0, full_path.size() - 1 );
  
  string new_data_loc = pathname( stripped_path );
  string new_filename = basename( stripped_path );
  
  entry.filename = new_filename;
  entry.data_loc = new_data_loc;
  entry.prefix = example_datafile[0];
  entry.files = files;
  
  return entry;
}

/// This function creates an ObservationEntry object for the file given.

ObservationEntry MakeObservationEntry( const string &single_file )
{
  ObservationEntry entry;
  
  entry.filename = basename( single_file );
  entry.data_loc = pathname( single_file );
  entry.prefix = "\\0";

  return entry;
}




/// This function iteratively examines a directory and calls itself to examine
/// any subdirectories. In each directory, it makes a list of all the filenames
/// that match any pattern we expect for an archive file. We may have several lists
/// for one directory if this directory contains cpsr2 data, so we add the filenames
/// to separate lists. Once we have listed all the files, we create an observation
/// for each filename (or list of filenames for cpsr2 data).

void ExamineDirectory( const string path )
{
  vector<string> rf_files;
  vector<string> cf_files;
  vector<string> m_fb_files;
  vector<string> n_fb_files;
  vector<string> m_cfb_files;
  vector<string> n_cfb_files;
  vector<string> r_cfb_files;
  
  boost::regex const rf_matcher(".*\\.rf");
  boost::regex const cf_matcher(".*\\.cf");
  boost::regex const m_fb_matcher("m.*\\.fb");
  boost::regex const n_fb_matcher("n.*\\.fb");
  boost::regex const m_cfb_matcher("m.*\\.cfb");
  boost::regex const n_cfb_matcher("n.*\\.cfb");
  boost::regex const r_cfb_matcher("r.*\\.cfb");

  vector<string> contents;
  dirglob( &contents, path + string( "/*" ) );
  vector<string>::iterator it;
  for( it = contents.begin(); it != contents.end(); it ++ )
  {
    string filename = basename( (*it) );
    if( file_is_directory( (*it).c_str() ) )
    {
      if( filename.find("_S") != string::npos )
        cerr << "ignoring scrunch directory " << (*it) << endl;
      else
        ExamineDirectory( (*it) );
    }
    else
    {
      if( boost::regex_match( filename, rf_matcher ) )
        rf_files.push_back( (*it) );
      else if( boost::regex_match( filename, cf_matcher ) )
        cf_files.push_back( (*it) );
      else if( boost::regex_match( filename, m_fb_matcher ) )
        m_fb_files.push_back( (*it) );
      else if( boost::regex_match( filename, n_fb_matcher ) )
        n_fb_files.push_back( (*it) );
      else if( boost::regex_match( filename, m_cfb_matcher ) )
        m_cfb_files.push_back( (*it) );
      else if( boost::regex_match( filename, n_cfb_matcher ) )
        n_cfb_files.push_back( (*it) );
      else if( boost::regex_match( filename, r_cfb_matcher ) )
        r_cfb_files.push_back( (*it) );
    }
  }
  
  for( it = cf_files.begin(); it != cf_files.end(); it ++ )
    db.CheckCalibration( MakeObservationEntry( (*it) ) );
  
  for( it = rf_files.begin(); it != rf_files.end(); it ++ )
    db.CheckObservation( MakeObservationEntry( (*it) ) );
  
  vector< vector<string> > cpsr2_observations;
  cpsr2_observations.push_back( m_cfb_files );
  cpsr2_observations.push_back( n_cfb_files );
  cpsr2_observations.push_back( r_cfb_files );
  cpsr2_observations.push_back( m_fb_files );
  cpsr2_observations.push_back( n_fb_files );
  
  vector< vector<string> >::iterator oit;
  for( oit = cpsr2_observations.begin(); oit != cpsr2_observations.end(); oit ++ )
  {
    if( (*oit).size() != 0 )
    {
      string first_file = (*oit).front();
      vector<string> dir_names = stringdecimate( first_file, "/" );
      
      string containing_dir = dir_names[dir_names.size()-3];
      if( containing_dir.substr( containing_dir.size()-2 ) == "_R" )
      {
	cerr << "we have a calibration " << endl;
        db.CheckCalibration( MakeObservationEntry( (*oit) ) );
      }
      else
      {
	cerr << dir_names[dir_names.size()-3] <<  " does not end with _R" << endl;
        db.CheckObservation( MakeObservationEntry( (*oit) ) );
      }
    }
  }
  
}


/// Connect to the database
/// Mark all the observations and cals as not present
/// iterate through all the directories given and their subdirectories
/// for every observation we find, check that its in the database
/// if its not in the database, add it and mark it as present
/// if its in the database, mark it as present
/// delete all the entries in the database that were not present.

int main( int argc, char *argv[] )
{
  try
  {
    db.Connect( database, db_hostname, db_username, db_password );
    
    db.MarkObservations();
    
    vector<string> target_directories;
    for( int i = 1; i < argc; i ++ )
      ExamineDirectory( string( argv[i] ) );

    db.PurgeMissingObservations();
    db.PurgeMissingCals();
  }
  catch( Exception e )
  {
    cerr << "The following exception was caught: ";
    cerr << e.what() << endl;
  }

  return 0;
}

